#include "system.h"
#include "video.h"
#include "background.h"
#include "dma.h"
#include "interrupts.h"
#include "sprite.h"
#include "ipc.h"

#define BUF_ATTRIBUTE0		(0x07000000)
#define BUF_ATTRIBUTE1		(0x07000002)
#define BUF_ATTRIBUTE2		(0x07000004)
#define BUF_ATTRIBUTE0_SUB	(0x07000400)
#define BUF_ATTRIBUTE1_SUB	(0x07000402)
#define BUF_ATTRIBUTE2_SUB	(0x07000404)

	.arm
	.align
	.global initSystem
	.global main

initSystem:
	bx lr

main:
	@ Setup the screens and the sprites
	
	bl InitScreens
	bl InitSprites
	bl InitData

	@ firstly, lets draw all the screen data ready for play
	@ and display the ship sprite
	
	bl WaitforVblank
	bl drawMapMain
	bl drawMapSub
	bl drawSFMapMain
	bl drawSFMapSub
	bl drawSBMapMain
	bl drawSBMapSub
	bl clearBG0
	bl drawScore
	bl drawSprite
	bl drawGetReadyText
	bl playInGameMusic

	bl WaitforFire
	mov r0,#1			@ just for checking (though this would NEVER be active at level start)
	str r0,powerUP
	
	bl clearBG0

@----------------------------@	
@ This is the MAIN game loop @
@----------------------------@
GameLoop:

	bl WaitforVblank
	@--------------------------------------------
	@ this code is executed offscreen
	@--------------------------------------------
		bl StarScroll		@ Scroll Stars (BG2,BG3)
		bl LevelDrift
		
		bl CheckEndOfLevel	@ Set Flag for end-of-level (use later to init BOSS)
		
		bl scrollMain		@ Scroll Level Data
		bl scrollSub		@ Main + Sub
	
		bl drawSprite		@ Move our craft (d-pad and A, perhaps check for pause and shut DS)

	bl WaitforNoblank
	@---------------------------------------------
	@ this code is executed during refresh
	@ this should give us a bit more time in vblank
	@---------------------------------------------
		bl moveShip
		bl moveBullets
		bl fireCheck
		bl addScore
		bl drawScore
	
	b GameLoop			@ our main loop
	
end:
	b end


@-------------------- From here on are all the subroutines and data -------------------@
fireCheck:			@ OUR CODE TO CHECK FOR FIRE (A) AND RELEASE A BULLET
	stmfd sp!, {r0-r6, lr}
	ldr r1,=REG_KEYINPUT
	ldr r2,[r1]
	tst r2,#BUTTON_A
	beq fireDown
		mov r0,#0
		str r0,firePress			@ set the flag to say fire has been released
		ldmfd sp!, {r0-r6, pc}
	fireDown:
		ldrb r0,powerUP				@ check if autofire is on
		cmp r0,#1
		beq initpowerUP				@ if so, lets use it
	
		ldr r0,firePress			@ load our pressed button check		
		cmp r0,#0					@ check if fire is released
		beq initFire				@ if it was, then lets see if we have a bullet spare?
		ldmfd sp!, {r0-r6, pc}
	initpowerUP:
		@ this is a little bit of code for auto fire
		@ add a little delay check in here to see if we are ready for another bullet
		@ if so, just continue the code!
		adrl r4,[powerUPDelay]		
		ldr r3,[r4]					@ r3=our delay
		add r3,#1					@ increment out counter
		cmp r3,#6					@ wait 8 refreshes before another bullet
		moveq r3,#0					@ if it is 8, zero the delay
		str r3,[r4]					@ put the result back
		beq initFire				@ ok, lets "fire one off"
		ldmfd sp!, {r0-r6, pc}		@ but, if not - we are done!
	initFire:
		mov r0,#1
		str r0,firePress
		@ Ok, now we need to see about initialising a bullet!!
		@ These are stored in our sprite table from position 1-4 (4 bullets)
		@ First thing is to see if we can fire a bullet.
		ldrb r0,powerUP
		cmp r0,#1
		moveq r0,#15				@ if powered up, we can allow 16 bullets
		movne r0,#3					@ if not, just the 4
		adrl r1,[spriteActive+4]	@ add 4 bytes as stored in words
		isbulletPossible:
			ldr r2,[r1,r0, lsl #2]	@ Multiplied by 4 as in words
			cmp r2,#0
			beq bulletPossible
			subs r0,#1
		bpl isbulletPossible
		ldmfd sp!, {r0-r6, pc}
	bulletPossible:
		@ ok, all we need to do now is set spriteActive for the bullet r0
		@ and set the image and coords for the bullet to start
		@ the bullet code will take care of the rest
		mov r2,#1
		adrl r1,[spriteActive+4]
		str r2,[r1,r0, lsl #2]		@ sprite is now active

		mov r2,#4					@ set the bullets speed!
		adrl r1,[spriteSpeed+4]
		str r2,[r1,r0, lsl #2]

		adrl r3,[spriteX]
		ldr r2,[r3]					@ our ships x coord
		adrl r4,[horizDrift]		@ we need to add our horizontal-
		ldr r4,[r4]					@ drift to the bullets X coord
		add r2,r4
		adrl r1,[spriteX+4]			@ store it in bullets x
		str r2,[r1,r0, lsl #2]		@ done!
		
		adrl r3,[spriteY]
		ldr r2,[r3]					@ our ships y coord
		sub r2,#8					@ move bullet up away from ship for a bit
		adrl r1,[spriteY+4]			@ store the result in bullets y
		str r2,[r1,r0, lsl #2]		@ done

		mov r2,#3					@ set r2 to our bullet image
		adrl r1,[spriteObj+4]		@ and store this on our sprite data
		str r2,[r1,r0, lsl #2]		@ done
		
		bl playBlasterSound
		
		@ bullet should now be ready to go!!!
		@ We can add the bullet movement code here, OR do it seperate
		@ along with the colision checks??? Hmmmm
	ldmfd sp!, {r0-r6, pc}
	
moveBullets:			@ OUR CODE TO MOVE THE ACTIVE BULLETS UP THE SCREEN
	stmfd sp!, {r0-r6, lr}
	mov r0,#15
	adrl r1,[spriteActive+4]		@ add 4 bytes as stored in words
	activeBloop:
		ldr r2,[r1,r0, lsl #2]		@ Multiplied by 4 as in words
		cmp r2,#0					@ check if bullet is active (1=yes)
		beq bulletDead
			@ Ok, this bullet is ACTIVE,
			@ so, lets move it, all we need to do is sub
			@ bullets speed from Ypos and check for off screen
			adrl r3,[spriteY+4]
			ldr r4,[r3,r0, lsl #2]
			
			adrl r5,[spriteSpeed+4]	@ Let us load r6 with the bullets speed
			ldr r6,[r5,r0, lsl #2]	@ this can be used for a power shot
				
			sub r4,r6				@ using r6 as a speed
			cmp r4,#360				@ this is our exit, any lower and we may shoot offscreen aliens
			bpl activeBstill
				mov r5,#0
				str r5,[r1,r0, lsl #2]
			activeBstill:
			str r4,[r3, r0, lsl #2]
		bulletDead:
		subs r0,#1
		bpl activeBloop	
	
	ldmfd sp!, {r0-r6, pc}

drawSprite:
	mov r8,#127				@ our counter for 128 sprites, do not think we need them all though
	adrl r4,[horizDrift] 	@ here we will set r4 as an adder for sprite offset
	ldr r4,[r4]				@ against our horizontal scroll
	
	SLoop:
	cmp r8,#0				@ if we are on our ship
	moveq r4,#0				@ then we dont need to add anything
							@ as our ship is not tied to the background
	
	@ If coord is <192 then plot on sub (upper)
	@ if >256 plot on main (lower)
	@ but, if on crossover, we need to plot 2 ships!!!
	@ one on each screen in the correct location!!
	@ Last section best commented!
	adrl r0,[spriteActive]
	ldr r1,[r0,r8, lsl #2]
	cmp r1,#1							@ Is sprite active?
	beq sprites_Drawn					@ if so, draw it!
	@ Kill sprite
		@ Kill sprite on both screens!!!!!
		@ note: read the setting - daft prat!!!	
	b sprites_Done
	sprites_Drawn:
	
	adrl r0,[spriteY]					@ Load Y coord
	ldr r1,[r0,r8,lsl #2]				@ add ,rX for offsets
	ldr r3,=383-32
	cmp r1,r3
	bmi sprites_Done
	ldr r3,=576+32
	cmp r1,r3
	bpl spriteY_Main_Done				@ Totally ON MAIN
	ldr r3,=576-32
	cmp r1,r3						@ Totally ON SUB
	bmi spriteY_Sub_Done

		@ The sprite is now between 2 screens and needs to be drawn to BOTH!
		@ Draw Y to MAIN screen (lower)
		ldr r0,=BUF_ATTRIBUTE0
		add r0,r8, lsl #3
		ldr r2, =(ATTR0_COLOR_16 | ATTR0_SQUARE)
		ldr r3,=576-32
		sub r1,r3
		cmp r1,#32
		addmi r1,#255
		sub r1,#32
		and r1,#0xff					@ Y is only 0-255
		orr r2,r1
		strh r2,[r0]
		@ Draw X to MAIN screen
		adrl r0,[spriteX]				@ get X coord mem space
		ldr r1,[r0,r8,lsl #2]			@ add ,Rx for offsets later!
		cmp r1,#64						@ if less than 64, this is off left of screen
		addmi r1,#512					@ convert coord for offscreen (32 each side)
		sub r1,#64						@ Take 64 off our X
		sub r1,r4						@ account for maps horizontal position
		ldr r3,=0x1ff					@ Make sure 0-512 only as higher would affect attributes
		ldr r0,=BUF_ATTRIBUTE1
		add r0,r8, lsl #3		
		ldr r2, =(ATTR1_SIZE_32)
		and r1,r3
		orr r2,r1
		strh r2,[r0]
			@ Draw Attributes
		ldr r0,=BUF_ATTRIBUTE2
		add r0,r8, lsl #3
		adrl r2,[spriteObj]
		ldr r3,[r2,r8, lsl #2]
		ldr r1,=(0 | ATTR2_PRIORITY(1) | ATTR2_PALETTE(0))
		orr r1,r3, lsl #4	
		str r1, [r0]

		adrl r0,[spriteY]				@ Load Y coord
		ldr r1,[r0,r8,lsl #2]	
	@ Sprite on top screen
	spriteY_Sub_Done:
		@ Draw sprite to SUB screen (r1 holds Y)
		ldr r0,=BUF_ATTRIBUTE0_SUB
		add r0,r8, lsl #3
		ldr r2, =(ATTR0_COLOR_16 | ATTR0_SQUARE)
		ldr r3,=384
		cmp r1,r3
		addmi r1,#255
		sub r1,r3
		and r1,#0xff					@ Y is only 0-255
		orr r2,r1
		strh r2,[r0]
		@ Draw X
		adrl r0,[spriteX]				@ get X coord mem space
		ldr r1,[r0,r8,lsl #2]			@ add ,rX for offsets
		cmp r1,#64						@ if less than 64, this is off left of screen
		addmi r1,#512					@ convert coord for offscreen (32 each side)
		sub r1,#64						@ Take 64 off our X
		sub r1,r4						@ account for maps horizontal position
		ldr r3,=0x1ff					@ Make sure 0-512 only as higher would affect attributes
		ldr r0,=BUF_ATTRIBUTE1_SUB		@
		add r0,r8, lsl #3
		ldr r2, =(ATTR1_SIZE_32)
		and r1,r3
		orr r2,r1
		strh r2,[r0]
			@ Draw Attributes
		ldr r0,=BUF_ATTRIBUTE2_SUB
		add r0,r8, lsl #3
		adrl r2,[spriteObj]
		ldr r3,[r2,r8, lsl #2]
		ldr r1,=(0 | ATTR2_PRIORITY(1) | ATTR2_PALETTE(0))
		orr r1,r3, lsl #4		
		str r1, [r0]

		@ Need to kill same sprite on MAIN screen - or do we???
		@ Seeing that for this to occur, the sprite is offscreen on MAIN!
	
		b sprites_Done
	
	spriteY_Main_Done:
		@ Draw sprite to MAIN
		ldr r0,=BUF_ATTRIBUTE0
		add r0,r8, lsl #3
		ldr r2, =(ATTR0_COLOR_16 | ATTR0_SQUARE)	@ These will not change in our game!
		ldr r3,=576-32				@ Calculate offsets
		sub r1,r3					@ R1 is STILL out Y coorrd
		cmp r1,#32					@ Acound for partial display
		addmi r1,#255				@ Modify if so (create a wrap)
		sub r1,#32					@ Take our sprite height off
		and r1,#0xff				@ Y is only 0-255
		orr r2,r1					@ Orr Y back with data in R2
		strh r2,[r0]				@ Store Y back
		@ Draw X
		adrl r0,[spriteX]			@ get X coord mem space
		ldr r1,[r0,r8,lsl #2]		@ add ,rX for offsets
		cmp r1,#64					@ if less than 64, this is off left of screen
		addmi r1,#512				@ convert coord for offscreen (32 each side)
		sub r1,#64					@ Take 64 off our X
		
		sub r1,r4					@ account for maps horizontal position
		
		ldr r3,=0x1ff				@ Make sure 0-512 only as higher would affect attributes
		ldr r0,=BUF_ATTRIBUTE1
		add r0,r8, lsl #3			@ Add offset (attribs in blocks of 8)
		ldr r2, =(ATTR1_SIZE_32)	@ Need a way to modify! 16384,32768,49152 = 16,32,64
		and r1,r3					@ kick out extranious on the Coord
		orr r2,r1					@ Stick the Coord and Data together
		strh r2,[r0]				@ and store them!
			@ Draw Attributes
		ldr r0,=BUF_ATTRIBUTE2		@ Find out Buffer Attribute
		add r0,r8, lsl #3			@ multiply by 8 to find location (in r0)
		adrl r2,[spriteObj]			@ Find our sprite to draw
		ldr r3,[r2,r8, lsl #2]		@ store in words (*2)
		ldr r1,=(0 | ATTR2_PRIORITY(1) | ATTR2_PALETTE(0))	@ Set Palette - do we need to keep doing this?
		orr r1,r3, lsl #4			@ Orr them together, R3 is mult by 16 (16 tiles in sprite *2)
		
		str r1, [r0]				@ Store it all back
		@ Need to kill same sprite on SUB screen - or do we???
		@ Seeing that for this to occur, the sprite is offscreen on SUB!
	sprites_Done:	
	subs r8,#1
	bpl SLoop

	mov r15,r14



addScore:
	stmfd sp!, {r0-r6, lr} 
	@ To use the score adder, just store the digits in 'adder' a byte for each and call this.
	@ the adder is cleared on exit to stop in keep counting digits.

	@ just to test!
	adrl r0,[adder+7]
	mov r1,#1
	strb r1,[r0]
	adrl r0,[adder+6]
	mov r1,#2
	strb r1,[r0]	

	mov r0,#7			@ r0 = start digit offset in score and adder
	adrl r3,[score]
	adrl r4,[adder]
	addLoop:
		ldrb r1,[r3,r0]
		ldrb r2,[r4,r0]
		add r5,r1,r2
		cmp r5,#10
		bmi addPassed
			sub r5,#10		
			sub r0,#1
			ldrb r6,[r3,r0]
			add r6,#1
			strb r6,[r3,r0]
			add r0,#1
		addPassed:
		strb r5,[r3,r0]
		mov r5,#0
		strb r5,[r4,r0]
		
		subs r0,#1
		bne addLoop
	ldmfd sp!, {r0-r6, pc}

clearBG0:
	stmfd sp!, {r0-r6, lr} 

	mov r0, #0
	ldr r1, =BG_MAP_RAM(15)
	ldr r2, =1024
	bl dmaFillWords
	ldr r1, =BG_MAP_RAM_SUB(17)
	bl dmaFillWords

	ldmfd sp!, {r0-r6, pc}
	
drawDigit:
	@ r0 - digit
	@ r1 - number
	stmfd sp!, {r2-r6, lr} 

	ldr r2, =BG_MAP_RAM(15)
	add r2, #1024+256+64+32+32
	mov r3, #4
	mov r4, #0
	mla r2, r0, r3, r2
	mov r3, #4
	mov r4, #2
	mla r3, r1, r3, r4
	strh r3, [r2]
	add r2, #2
	add r3, #1
	strh r3, [r2]
	add r2, #63
	add r3, #1
	strh r3, [r2]
	add r2, #1
	add r3, #1
	strh r3, [r2]

	ldmfd sp!, {r2-r6, pc}

drawScore:
	stmfd sp!, {r0-r6, lr} 
	mov r0,#7
	adrl r2,[score]
	digitLoop:
		ldrb r1,[r2,r0]
		bl drawDigit
		subs r0,#1
	bpl digitLoop
	ldmfd sp!, {r0-r6, pc}
	
drawGetReadyText:
	stmfd sp!, {r0-r6, lr} 
	
	ldr r0, =BG_MAP_RAM(15)			@ make r0 a pointer to screen memory bg bitmap sub address
	ldr r1, =(512+22)
	add r0, r1
	ldr r1, =getReadyText			@ Get address of text characters to draw
	mov r2, #10						@ 70 characters to draw
	
mapLoop:
	ldrb r3,[r1],#1					@ Read r1 [text] and add 1 to [text] offset
	sub r3, #32						@ ASCII character - 32 to get tile offset
	add r3, #42						@ Skip 42 tiles (score digits)
	strh r3, [r0], #2				@ Write the tile number to our 32x32 map and move along
	subs r2, r2, #1					@ Move along one
	bne mapLoop						@ And loop back until done
	
	ldmfd sp!, {r0-r6, pc}

LevelDrift:
	ldrb r0, horizDrift
	ldr r1, =REG_BG1HOFS			@ Load our horizontal scroll register for BG1 on the main screen
	ldr r2, =REG_BG1HOFS_SUB		@ Load our horizontal scroll register for BG1 on the sub screen
	strh r0,[r1]
	strh r0,[r2]
	mov r15,r14

StarScroll:
	stmfd sp!, {r0-r6, lr} 
	ldr r1,delaySF							@ Delay Starfront for every other update
	subs r1, #1
	bne noscrollSF
		bl scrollSFMain
		bl scrollSFSub
		mov r1,#2
	noscrollSF:
	str r1,delaySF	
	ldr r1,delaySB							@ Delay Starback for every 4 updates
	subs r1,#1
	bne noscrollSB
		bl scrollSBMain
		bl scrollSBSub
		mov r1,#4
	noscrollSB:
	str r1,delaySB
	ldmfd sp!, {r0-r6, pc}
	
CheckEndOfLevel:
	stmfd sp!, {r0-r6, lr} 
	ldrb r0,LevelEnd
	cmp r0,#0
	bne LevelPlay
		ldr r0,yposSub						@ Ypos is the Y position in the map data
		ldr r1,vofsSub
		orr r0, r1
		cmp r0, #0							@ are we at ZERO - top of the map?
		bne LevelPlay						@ If so, and scroll is 0 also - Stop Main Scroll!
			mov r0,#1
			strb r0,LevelEnd
	LevelPlay:
	ldmfd sp!, {r0-r6, pc}

WaitforVblank:
	stmfd sp!, {r0-r6, lr} 
	
	ldrb r0,LevelEnd
	cmp r0,#0
	beq LevelBlank
		@ Do our effect for the boss battle!
		ldr r3,=sinus1					@ address of sine
		ldr r5,=REG_BG2HOFS_SUB			@ Preset memory adresses for Horiz Scroll
		ldr r6,=REG_BG2HOFS
		ldr r7,=REG_BG3HOFS_SUB
		ldr r8,=REG_BG3HOFS
		mov r1,#0						@ r1 is our sine/Raster counter
		ldr r0, =REG_VCOUNT
		holdonmate:
			ldrh r2, [r0]				@ r2 = raster position <-- (This fixed the problem on h/w -HK)				
			cmp r2,r1					@ check if we are at our raster pos?
		bne holdonmate					@ if not, keep waiting
			ldrb r4,[r3,r1]			@ load r4 with sine1 position plus our raster position offest
			strh r4,[r5]	
			strh r4,[r6]
			strh r4,[r7]
			strh r4,[r8]
			add r1,#1					@ add 1 to the raster count (the raster number we are modifying)
			cmp r1,#193					@ check if we are down to our end position
		bne holdonmate					@ if not, let us do another line
		ldmfd sp!, {r0-r6, pc}
	
	LevelBlank:
	ldr r0, =REG_VCOUNT
	waitVBlankSUB:									
	ldrh r1,[r0]						@ read REG_VCOUNT into r2
	cmp r1, #193						@ 193 is, of course, the first scanline of vblank
	bne waitVBlankSUB					@ loop if r2 is not equal to (NE condition) 193
	ldmfd sp!, {r0-r6, pc}
	
WaitforNoblank:
	stmfd sp!, {r0-r6, lr} 
	ldr r0, =REG_VCOUNT
	waitVBlankNO:									
	ldrh r1, [r0]						@ read REG_VCOUNT into r2
	cmp r1, #230	
	bne waitVBlankNO
	ldmfd sp!, {r0-r6, pc}

scrollMain:
	stmfd sp!, {r0-r6, lr}
	ldrb r0,LevelEnd
	cmp r0,#0
	bne scrollDone
	adrl r2,[vofsMain]
	ldrh r1, [r2]					@ Load r2 with the scroll register
	cmp r1, #0						@ has our scroll register reached zero?
	beq resetScrollMain				@ Then reset our scroller
	cmp r1, #512					@ Has our scroller reached 255 lines?
	beq scrollMapMain				@ If so, time to scroll the map
	subs r1, r1, #1					@ move up a pixel
	ldr r3, =REG_BG1VOFS			@ R2 is the memory adress for the main scroll
	strh r1, [r2]
	strh r1, [r3]					@ write our scroll counter into REG_BG0VOFS main screen	
	Noscrollmain:
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
scrollSub:
	stmfd sp!, {r0-r6, lr}
	ldrb r0,LevelEnd
	cmp r0,#0
	bne scrollDone
	adrl r2, [vofsSub]
	ldrh r1, [r2]					@ Load r2 with the scroll register
	cmp r1, #0						@ has our scroll register reached zero?
	beq resetScrollSub				@ Then reset our scroller
	cmp r1, #512					@ Has our scroller reached 255 lines?
	beq scrollMapSub				@ If so, time to scroll the map
	subs r1, r1, #1					@ move up a pixel
	ldr r3, =REG_BG1VOFS_SUB		@ R2 is the memory adress for the main scroll
	strh r1, [r2]
	strh r1, [r3]					@ write our scroll counter into REG_BG0VOFS main screen
	NoscrollSub:
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
scrollSFMain:
	stmfd sp!, {r0-r6, lr} 

	adrl r2, [vofsSFMain]
	ldrh r1, [r2]					@ Load r2 with the scroll register
	cmp r1, #0						@ has our scroll register reached zero?
	beq resetScrollSFMain			@ Then reset our scroller
	cmp r1, #512					@ Has our scroller reached 255 lines?
	beq scrollSFMapMain				@ If so, time to scroll the map
	subs r1, r1, #1					@ move up a pixel
	ldr r3, =REG_BG2VOFS			@ R2 is the memory adress for the main scroll
	strh r1, [r2]
	strh r1, [r3]					@ write our scroll counter into REG_BG0VOFS main screen
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return

scrollSFSub:
	stmfd sp!, {r0-r6, lr}	

	adrl r2, [vofsSFSub]
	ldrh r1, [r2]					@ Load r2 with the scroll register
	cmp r1, #0						@ has our scroll register reached zero?
	beq resetScrollSFSub			@ Then reset our scroller
	cmp r1, #512					@ Has our scroller reached 255 lines?
	beq scrollSFMapSub				@ If so, time to scroll the map
	subs r1, r1, #1					@ move up a pixel
	ldr r3, =REG_BG2VOFS_SUB		@ R2 is the memory adress for the main scroll
	strh r1, [r2]
	strh r1, [r3]					@ write our scroll counter into REG_BG0VOFS main screen
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
scrollSBMain:
	stmfd sp!, {r0-r6, lr} 	

	adrl r2, [vofsSBMain]
	ldrh r1, [r2]					@ Load r2 with the scroll register
	cmp r1, #0						@ has our scroll register reached zero?
	beq resetScrollSBMain			@ Then reset our scroller
	cmp r1, #512					@ Has our scroller reached 255 lines?
	beq scrollSBMapMain				@ If so, time to scroll the map
	subs r1, r1, #1					@ move up a pixel
	ldr r3, =REG_BG3VOFS			@ R2 is the memory adress for the main scroll
	strh r1, [r2]
	strh r1, [r3]					@ write our scroll counter into REG_BG0VOFS main screen	
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
scrollSBSub:
	stmfd sp!, {r0-r6, lr}	

	adrl r2, [vofsSBSub]
	ldrh r1, [r2]					@ Load r2 with the scroll register
	cmp r1, #0						@ has our scroll register reached zero?
	beq resetScrollSBSub			@ Then reset our scroller
	cmp r1, #512					@ Has our scroller reached 255 lines?
	beq scrollSBMapSub				@ If so, time to scroll the map
	subs r1, r1, #1					@ move up a pixel
	ldr r3, =REG_BG3VOFS_SUB		@ R2 is the memory adress for the main scroll
	strh r1, [r2]
	strh r1, [r3]					@ write our scroll counter into REG_BG0VOFS main screen
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
resetScrollMain:
	stmfd sp!, {r0-r6, lr} 
	adrl r0, [vofsMain]
	ldr r1, =REG_BG1VOFS			@ R1 is the memory adress for the sub screen scroll
	ldr r2, =255					@ Now lets restore the scroll register back to 255
	strh r2, [r0]					@ and store in
	strh r2, [r1]					@ and store in
	bl scrollMapMain
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
resetScrollSub:
	stmfd sp!, {r0-r6, lr} 
	adrl r0, [vofsSub]
	ldr r1, =REG_BG1VOFS_SUB		@ R1 is the memory adress for the sub screen scroll
	ldr r2, =255					@ Now lets restore the scroll register back to 255
	strh r2, [r0]					@ and store in
	strh r2, [r1]					@ and store in
	bl scrollMapSub
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
resetScrollSFMain:
	stmfd sp!, {r0-r6, lr} 

	adrl r0, [vofsSFMain]
	ldr r1, =REG_BG2VOFS			@ R1 is the memory adress for the sub screen scroll
	ldr r2, =255					@ Now lets restore the scroll register back to 255
	strh r2, [r0]					@ and store in
	strh r2, [r1]					@ and store in
	bl scrollSFMapMain
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return

resetScrollSFSub:
	stmfd sp!, {r0-r6, lr} 

	adrl r0, [vofsSFSub]
	ldr r1, =REG_BG2VOFS_SUB		@ R1 is the memory adress for the sub screen scr
	ldr r2, =255					@ Now lets restore the scroll register back to 255
	strh r2, [r0]					@ and store in
	strh r2, [r1]					@ and store in
	bl scrollSFMapSub
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return

resetScrollSBMain:
	stmfd sp!, {r0-r6, lr} 
	
	adrl r0, [vofsSBMain]
	ldr r1, =REG_BG3VOFS			@ R1 is the memory adress for the sub screen scroll
	ldr r2, =255					@ Now lets restore the scroll register back to 255
	strh r2, [r0]					@ and store in
	strh r2, [r1]					@ and store in
	bl scrollSBMapMain
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
resetScrollSBSub:
	stmfd sp!, {r0-r6, lr} 
	
	adrl r0, [vofsSBSub]
	ldr r1, =REG_BG3VOFS_SUB		@ R1 is the memory adress for the sub screen scroll
	ldr r2, =255					@ Now lets restore the scroll register back to 255
	strh r2, [r0]					@ and store in
	strh r2, [r1]					@ and store in
	bl scrollSBMapSub
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return

scrollMapMain:
	stmfd sp!, {r0-r6, lr} 
	adrl r0, [yposMain]				@ grab ypos memory adress
	ldr r1, [r0]					@ r3 = ypos
	sub r1, #64						@ lets go up one block (64 tiles) on the map
	strh r1, [r0]					@ and put the value back for later
	bl drawMapMain
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
scrollMapSub:
	stmfd sp!, {r0-r6, lr} 
	adrl r0, [yposSub]				@ grab ypos memory adress
	ldr r1, [r0]					@ r3 = ypos
	sub r1, #64						@ lets go up one block (64 tiles) on the map
	strh r1, [r0]					@ and put the value back for later
	bl drawMapSub
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return

scrollSFMapMain:
	stmfd sp!, {r0-r6, lr} 
	adrl r0, [yposSFMain]			@ grab ypos memory adress
	ldr r1, [r0]					@ r3 = ypos
	
	cmp r1,#0						@ If we are at the top, lets go back to the
	moveq r1,#832					@ bottom of the map!

	sub r1, #64						@ lets go up one block (64 tiles) on the map
	strh r1, [r0]					@ and put the value back for later
	bl drawSFMapMain
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return

scrollSFMapSub:
	stmfd sp!, {r0-r6, lr} 
	adrl r0, [yposSFSub]			@ grab ypos memory adress
	ldr r1, [r0]					@ r3 = ypos
	
	cmp r1,#0						@ If we are at the top, lets go back to the
	moveq r1,#832					@ bottom of the map!
	
	sub r1, #64						@ lets go up one block (64 tiles) on the map
	strh r1, [r0]					@ and put the value back for later
	bl drawSFMapSub
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
scrollSBMapMain:
	stmfd sp!, {r0-r6, lr} 
	adrl r0, [yposSBMain]			@ grab ypos memory adress
	ldr r1, [r0]					@ r3 = ypos

	cmp r1,#0						@ If we are at the top, lets go back to the
	moveq r1,#832					@ bottom of the map!

	sub r1, #64						@ lets go up one block (64 tiles) on the map
	strh r1, [r0]					@ and put the value back for later
	bl drawSBMapMain
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
scrollSBMapSub:
	stmfd sp!, {r0-r6, lr} 
	adrl r0, [yposSBSub]				@ grab ypos memory adress
	ldr r1, [r0]					@ r3 = ypos

	cmp r1,#0						@ If we are at the top, lets go back to the
	moveq r1,#832					@ bottom of the map!

	sub r1, #64						@ lets go up one block (64 tiles) on the map
	strh r1, [r0]					@ and put the value back for later
	bl drawSBMapSub
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return
	
scrollDone:
	ldmfd sp!, {r0-r6, pc} 		@ restore rgisters and return

drawMapMain:
	@ Draw Main Screen
	stmfd sp!, {r0-r6, lr}
	ldr r0, =Level1Map
	ldr r1, =BG_MAP_RAM(19)
	adrl r4, [yposMain]
	ldr r2, [r4]					@ load y2 with ypos (map reference)
	add r0, r2, lsl #6				@ ycoord * 64 added to screenbase
	ldr r2, =8192					@ 64x64x2
	bl dmaCopy
	ldmfd sp!, {r0-r6, pc} 		@ restore registers and return
drawMapSub:
	stmfd sp!, {r0-r6, lr}
	@ Draw Sub screen
	ldr r0, =Level1Map
	ldr r1, =BG_MAP_RAM_SUB(21)
	adrl r4, [yposSub]
	ldr r2, [r4]					@ load y2 with ypos (map reference)
	add r0, r2, lsl #6				@ ycoord * 64 added to screenbase
	ldr r2, =8192					@ 64x64x2
	bl dmaCopy
	ldmfd sp!, {r0-r6, pc} 		@ restore registers and return
drawSFMapMain:
	@ Draw Main Screen
	stmfd sp!, {r0-r6, lr}
	ldr r0, =StarFrontMap
	ldr r1, =BG_MAP_RAM(23)
	adrl r4, [yposSFMain]
	ldr r2, [r4]					@ load y2 with ypos (map reference)
	add r0, r2, lsl #5				@ ycoord * 64 added to screenbase
	ldr r2, =4096					@ 32x64x2
	bl dmaCopy
	ldmfd sp!, {r0-r6, pc} 		@ restore registers and return
drawSFMapSub:
	stmfd sp!, {r0-r6, lr}
	@ Draw Sub screen
	ldr r0, =StarFrontMap
	ldr r1, =BG_MAP_RAM_SUB(25)
	adrl r4, [yposSFSub]
	ldr r2, [r4]					@ load y2 with ypos (map reference)
	add r0, r2, lsl #5				@ ycoord * 64 added to screenbase
	ldr r2, =4096					@ 32x64x2
	bl dmaCopy
	ldmfd sp!, {r0-r6, pc} 		@ restore registers and return
drawSBMapMain:
	@ Draw Main Screen
	stmfd sp!, {r0-r6, lr}
	ldr r0, =StarBackMap
	ldr r1, =BG_MAP_RAM(27)
	adrl r4, [yposSBMain]
	ldr r2, [r4]					@ load y2 with ypos (map reference)
	add r0, r2, lsl #5				@ ycoord * 64 added to screenbase
	ldr r2, =4096					@ 32x64x2
	bl dmaCopy
	ldmfd sp!, {r0-r6, pc} 		@ restore registers and return
drawSBMapSub:
	stmfd sp!, {r0-r6, lr}
	@ Draw Sub screen
	ldr r0, =StarBackMap
	ldr r1, =BG_MAP_RAM_SUB(29)
	adrl r4, [yposSBSub]
	ldr r2, [r4]					@ load y2 with ypos (map reference)
	add r0, r2, lsl #5				@ ycoord * 64 added to screenbase
	ldr r2, =4096					@ 32x64x2
	bl dmaCopy
	ldmfd sp!, {r0-r6, pc} 		@ restore registers and return

InitScreens:
	stmfd sp!, {r0-r6, lr}
	ldr r0, =REG_POWERCNT
	ldr r1, =POWER_ALL_2D			@ All power on
	str r1, [r0]
 
	mov r0, #REG_DISPCNT			@ Main screen to Mode 0 with BG2 active
	ldr r1, =(MODE_0_2D | DISPLAY_SPR_ACTIVE | DISPLAY_SPR_1D_LAYOUT | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE | DISPLAY_BG2_ACTIVE | DISPLAY_BG3_ACTIVE)
	str r1, [r0]
	
	ldr r0, =REG_DISPCNT_SUB		@ Sub screen to Mode 0 with BG0 active
	ldr r1, =(MODE_0_2D | DISPLAY_SPR_ACTIVE | DISPLAY_SPR_1D_LAYOUT | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE | DISPLAY_BG2_ACTIVE | DISPLAY_BG3_ACTIVE)
	str r1, [r0]
 
	ldr r0, =VRAM_A_CR				@ Set VRAM A to be main bg address 0x06000000
	ldr r1, =(VRAM_ENABLE | VRAM_A_MAIN_BG_0x06000000)
	strb r1, [r0]

	ldr r0, =VRAM_B_CR				@ Use this for sprite data
	ldr r1, =(VRAM_ENABLE | VRAM_A_MAIN_SPRITE)
	strb r1, [r0]
	
	ldr r0, =VRAM_C_CR				@ Set VRAM C to be sub bg address 0x06200000
	ldr r1, =(VRAM_ENABLE | VRAM_C_SUB_BG_0x06200000)
	strb r1, [r0]
	
	ldr r0, =VRAM_D_CR				@ Use this for sprite data
	ldr r1, =(VRAM_ENABLE | VRAM_D_SUB_SPRITE)
	strb r1, [r0]

	ldr r0, =REG_BG0CNT				@ Set main screen BG0 format to be 64x64 tiles at base address
	ldr r1, =(BG_COLOR_16 | BG_32x32 | BG_MAP_BASE(15) | BG_TILE_BASE(0) | BG_PRIORITY(0))
	strh r1, [r0]
	ldr r0, =REG_BG0CNT_SUB			@ Set sub screen BG0 format to be 64x64 tiles at base address
	ldr r1, =(BG_COLOR_16 | BG_32x32 | BG_MAP_BASE(17) | BG_TILE_BASE(0) | BG_PRIORITY(0))
	strh r1, [r0]
	
	ldr r0, =REG_BG1CNT				@ Set main screen BG0 format to be 64x64 tiles at base address
	ldr r1, =(BG_COLOR_16 | BG_64x64 | BG_MAP_BASE(19) | BG_TILE_BASE(1) | BG_PRIORITY(1))
	strh r1, [r0]
	ldr r0, =REG_BG1CNT_SUB			@ Set sub screen BG0 format to be 64x64 tiles at base address
	ldr r1, =(BG_COLOR_16 | BG_64x64 | BG_MAP_BASE(21) | BG_TILE_BASE(1) | BG_PRIORITY(1))
	strh r1, [r0]
	
	ldr r0, =REG_BG2CNT				@ Set main screen BG0 format to be 64x64 tiles at base address
	ldr r1, =(BG_COLOR_16 | BG_32x64 | BG_MAP_BASE(23) | BG_TILE_BASE(2) | BG_PRIORITY(2))
	strh r1, [r0]
	ldr r0, =REG_BG2CNT_SUB			@ Set sub screen BG0 format to be 64x64 tiles at base address
	ldr r1, =(BG_COLOR_16 | BG_32x64 | BG_MAP_BASE(25) | BG_TILE_BASE(2) | BG_PRIORITY(2))
	strh r1, [r0]

	ldr r0, =REG_BG3CNT				@ Set main screen BG3 format to be 64x64 tiles at base address
	ldr r1, =(BG_COLOR_256 | BG_32x64 | BG_MAP_BASE(27) | BG_TILE_BASE(5) | BG_PRIORITY(3))
	strh r1, [r0]
	ldr r0, =REG_BG3CNT_SUB			@ Set sub screen BG3 format to be 64x64 tiles at base address
	ldr r1, =(BG_COLOR_256 | BG_32x64 | BG_MAP_BASE(29) | BG_TILE_BASE(5) | BG_PRIORITY(3))
	strh r1, [r0]
	
	@ Load the palette into the palette subscreen area and main

	ldr r0, =StarBackPal
	ldr r1, =BG_PALETTE
	ldr r2, =512
	bl dmaCopy
	ldr r1, =BG_PALETTE_SUB
	bl dmaCopy
	
	@ Write the tile data to VRAM Level BG1

	ldr r0, =Level1Tiles
	ldr r1, =BG_TILE_RAM(1)
	ldr r2, =32768
	bl dmaCopy
	ldr r1, =BG_TILE_RAM_SUB(1)
	bl dmaCopy
	
	@ Write the tile data to VRAM FrontStar BG2

	ldr r0, =StarFrontTiles
	ldr r1, =BG_TILE_RAM(2)
	ldr r2, =640
	bl dmaCopy
	ldr r1, =BG_TILE_RAM_SUB(2)
	bl dmaCopy

	@ Write the tile data to VRAM BackStar BG3

	ldr r0, =StarBackTiles
	ldr r1, =BG_TILE_RAM(5)
	ldr r2, =34880
	bl dmaCopy
	ldr r1, =BG_TILE_RAM_SUB(5)
	bl dmaCopy

	ldr r0, =ScoreTiles
	ldr r1, =BG_TILE_RAM(0)
	ldr r2, =1344
	bl dmaCopy
	ldr r1, =BG_TILE_RAM_SUB(0)
	bl dmaCopy
	
	ldr r0, =FontTiles
	ldr r1, =BG_TILE_RAM(0)
	add r1, #1344
	ldr r2, =3072
	bl dmaCopy
	ldr r1, =BG_TILE_RAM_SUB(0)
	bl dmaCopy

	@ Our horizontal centre routine
	
	ldr r0, =REG_BG1HOFS			@ Load our horizontal scroll register for BG1 on the main screen
	ldr r1, =REG_BG1HOFS_SUB		@ Load our horizontal scroll register for BG1 on the sub screen
	ldr r2, =REG_BG2HOFS			@ Load our horizontal scroll register for BG2 on the main screen
	ldr r3, =REG_BG2HOFS_SUB		@ Load our horizontal scroll register for BG2 on the sub screen
	ldr r4, =REG_BG3HOFS			@ Load our horizontal scroll register for BG3 on the main screen
	ldr r5, =REG_BG3HOFS_SUB		@ Load our horizontal scroll register for BG3 on the sub screen
	mov r6, #32						@ Offset the horizontal scroll register by 32 pixels to centre the map
	strh r6, [r0]					@ Write our offset value to REG_BG1HOFS
	strh r6, [r1]					@ Write our offset value to REG_BG1HOFS_SUB
	mov r6,#0
	strh r6, [r2]					@ Write our offset value to REG_BG2HOFS
	strh r6, [r3]					@ Write our offset value to REG_BG2HOFS_SUB
	strh r6, [r4]					@ Write our offset value to REG_BG3HOFS
	strh r6, [r5]					@ Write our offset value to REG_BG3HOFS_SUB

		
	@ Our vertical scrolling routine

	ldr r0, =REG_BG1VOFS			@ Load our vertical scroll register for BG1 on the main screen
	adrl r2, [vofsMain]				@ Set our scroll counter to start main screen
	ldrh r2, [r2]
	strh r2, [r0]					@ Load the value into the scroll register

	ldr r0, =REG_BG1VOFS_SUB		@ Load our vertical scroll register for BG1 on the sub screen
	adrl r2, [vofsSub]				@ Set our scroll counter to start sub screen
	ldrh r2, [r2]
	strh r2, [r0]					@ Load the value into the scroll register
	
	ldr r0, =REG_BG2VOFS			@ Load our vertical scroll register for BG2 on the main screen
	adrl r2, [vofsSFMain]			@ Set our scroll counter to start main screen
	ldrh r2, [r2]
	strh r2, [r0]					@ Load the value into the scroll register

	ldr r0, =REG_BG2VOFS_SUB		@ Load our vertical scroll register for BG2 on the sub screen
	adrl r2, [vofsSFSub]			@ Set our scroll counter to start sub screen
	ldrh r2, [r2]
	strh r2, [r0]					@ Load the value into the scroll register

	ldr r0, =REG_BG3VOFS			@ Load our vertical scroll register for BG3 on the main screen
	adrl r2, [vofsSBMain]			@ Set our scroll counter to start main screen
	ldrh r2, [r2]
	strh r2, [r0]					@ Load the value into the scroll register

	ldr r0, =REG_BG3VOFS_SUB		@ Load our vertical scroll register for BG3 on the sub screen
	adrl r2, [vofsSBSub]			@ Set our scroll counter to start sub screen
	ldrh r2, [r2]
	strh r2, [r0]	
	
	ldmfd sp!, {r0-r6, pc} 

InitSprites:
	stmfd sp!, {r0-r6, lr} 

	@ Load the palette into the palette subscreen area and main

	ldr r0, =SpritesPal
	ldr r1, =SPRITE_PALETTE
	ldr r2, =512
	bl dmaCopy
	ldr r1, =SPRITE_PALETTE_SUB
	bl dmaCopy

	@ Write the tile data to VRAM

	ldr r0, =SpritesTiles
	ldr r1, =SPRITE_GFX
	ldr r2, =65536
	bl dmaCopy
	ldr r1, =SPRITE_GFX_SUB
	bl dmaCopy
	
	@ Clear the OAM (disable all 128 sprites for both screens)

	ldr r0, =ATTR0_DISABLED			@ Set OBJ_ATTRIBUTE0 to ATTR0_DISABLED
	ldr r1, =OAM
	ldr r2, =1024					@ 3 x 16bit attributes + 16 bit filler = 8 bytes x 128 entries in OAM
	bl dmaFillWords
	ldr r1, =OAM_SUB
	bl dmaFillWords
	
	ldr r0,=0
	ldr r1,=spriteX
	ldr r2,=128+128+128+128+128
	bl dmaFillWords

	ldmfd sp!, {r0-r6, pc} 
	
WaitforFire:
	@ Messy bit just to wait for button to start
	@ This is a delay that can be skipped by FIRE
	@ Not great, but serves its purpose for now
	@ this will need to update music at later stage also
	
	stmfd sp!, {r0-r6, lr} 
	mov r4,#200
	buttpause:
	
		bl WaitforVblank
	
		bl StarScroll

		bl WaitforNoblank
	
		ldr r1,=REG_KEYINPUT
		ldr r2,[r1]
		tst r2,#BUTTON_A
		beq butta
		subs r4,#1
		bne buttpause
	butta:	
	
	ldmfd sp!, {r0-r6, pc} 
	
moveShip:
	stmfd sp!, {r0-r8, lr} 
	ldr r5,=spriteY
	ldr r6,=spriteX
	ldr r1,=REG_KEYINPUT		@ Read the keys!
	ldrb r7,powerUP				@ check if we are "Powered up"
	cmp r7, #1					@ a 1 signals a power up
	moveq r7,#4					@ if so, move 4 pixels per refresh
	movne r7,#2					@ if not, move our standard 2
	
								@ r7 now holds our ship speed
	ldr r2,[r1]					@ R2 is the input (except X and Y handled by ARM7)
	tst r2,#BUTTON_UP			@ UP (and with value to isolate direction)
	bne dircheck1
	@ Up code
		ldr r8,[r5]
		subs r8,#2
		movmi r8,#0
		str r8,[r5]
		b dircheck2
	dircheck1:
	ldr r2,[r1]					@ R2 is the input (except X and Y handled by ARM7)
	tst r2,#BUTTON_DOWN			@ DOWN (and with value to isolate direction)
	bne dircheck2
	@ Down code
		ldr r8,[r5]
		adds r8,#2
		cmp r8,#736
		movpl r8,#736
		str r8,[r5]
	dircheck2:
	ldr r2,[r1]					@ R2 is the input (except X and Y handled by ARM7)
	tst r2,#BUTTON_LEFT			@ LEFT (and with value to isolate direction)
	bne dircheck3
	@ Left code
		ldrb r8,horizDrift
		cmp r8,#0
		beq leftmove
			sub r8,r7, lsr #1
			strb r8,horizDrift
			ldr r8,[r6]	
			subs r8,r7, lsr #1
			cmp r8,#64
			movmi r8,#64
			b dircheck2pass
		leftmove:
		ldr r8,[r6]
		subs r8,r7
		cmp r8,#64
		movmi r8,#64
		dircheck2pass:
		str r8,[r6]
		b dircheck4
	dircheck3:
	ldr r2,[r1]					@ R2 is the input (except X and Y handled by ARM7)
	tst r2,#BUTTON_RIGHT		@ RIGHT (and with value to isolate direction)
	bne dircheck4
	@ Right code
		ldrb r8,horizDrift
		cmp r8,#64
		beq rightmove
			add r8,r7, lsr #1
			strb r8,horizDrift
			ldr r8,[r6]
			mov r4,#256
			add r4,#32
			adds r8,r7,lsr #1
			cmp r8,r4
			movpl r8,r4
			b dircheck3pass
		rightmove:
		ldr r8,[r6]
		mov r4,#256
		add r4,#32		
		adds r8,r7
		cmp r8,r4			@ 256-sprite width
		movpl r8,r4
		dircheck3pass:
		str r8,[r6]
	dircheck4:
	
	ldmfd sp!, {r0-r8, pc} 


drawShip:
	stmfd sp!, {r0-r2, lr} 
	
	@ Update the Ship with the valid position
	ldr r0, =OBJ_ATTRIBUTE0(0)					@ Plot Y Coord
	ldr r1,shipY
	and r1, #0x00ff								@ And our mask
	orr r1,#(ATTR0_COLOR_16 | ATTR0_SQUARE)		@ Or our attribs
	strh r1, [r0]								@ Write out 16 bit register (you can't write 8 bits to a 16 bit register)
	ldr r0, =OBJ_ATTRIBUTE1(0)					@ Plot x Coord
	ldr r1,shipX
	ldr r2, =0x01ff								@ Load our mask
	and r1, r2									@ And our mask
	orr r1,#(ATTR1_SIZE_32)						@ Or in our attribs
	strh r1, [r0]								@ Write to the 16 bit register
	ldr r0, =OBJ_ATTRIBUTE2(0)
	ldr r1, =(0 | ATTR2_PRIORITY(1) | ATTR2_PALETTE(0))
	strh r1, [r0]
	
	ldmfd sp!, {r0-r2, pc} 
	
Divide:
	@ R1 to hold nuber to divide, r2 hold divisor. R0 returns result (hopefully)
	@ Thought this may com in handy - perhaps not?
	stmfd sp!, {r3-r6, lr} 
	mov r0,#0
	mov r3,#1	@ set bit 0 for test in shift
	divl1:
		cmp r2,r1
		movls r2,r2,lsl #1	@ shift r2 till it is same as r1
		movls r3,r3,lsl #1	@ shift r3 (this is our count flag)
	bls divl1
	divl2:
		cmp r1,r2
		subcs r1,r1,r2
		addcs r0,r0,r3
		movs r3,r3,lsr #1
		movcc r2,r2,lsr #1
	bcc divl2
	ldmfd sp!, {r3-r6, pc} 
	
InitData:
	stmfd sp!, {r0-r1, lr} 
	mov r1,#0	
	str r1,LevelEnd			@ Flag is SET for end of level
	str r1,powerUP			@ set to one for autofire
	str r1,powerUPDelay		@ clear the fire delay
	mov r1, #1
	str r1,firePress		@ Set fire press so fire to start does not fire a bullet
	str r1,spriteActive		@ make sure our ship is visible
	mov r1,#176
	str r1,spriteX			@ our ships initial X coord
	mov r1,#716
	str r1,spriteY			@ our ships initial Y coord
	mov r1,#32
	strb r1,horizDrift		@ Drift on levels horizontal - centre to start level
	ldmfd sp!, {r0-r1, pc}
	
playInGameMusic:
	stmfd sp!, {r0-r1, lr}

	ldr r0, =IPC_SOUND_LEN(0)		@ Get the IPC sound length address
	ldr r1, =851393					@ Get the sample size
	str r1, [r0]					@ Write the sample size
	
	ldr r0, =IPC_SOUND_DATA(0)		@ Get the IPC sound data address
	ldr r1, =ingame_wav				@ Get the sample address
	str r1, [r0]					@ Write the value
	
	ldmfd sp!, {r0-r1, pc} 		@ restore rgisters and return
	
playBlasterSound:
	stmfd sp!, {r0-r2, lr}

	ldr r0, =IPC_SOUND_LEN(1)		@ Get the IPC sound length address
	ldr r1, =5170					@ Get the sample size
	str r1, [r0]					@ Write the sample size
	
	ldr r0, =IPC_SOUND_DATA(1)		@ Get the IPC sound data address
	ldr r1, =blaster_raw			@ Get the sample address
	str r1, [r0]					@ Write the value
	
	ldmfd sp!, {r0-r2, pc} 		@ restore rgisters and return
	
	.pool
	.align 4
	pressme:
	.word 0


firePress:
	.word 0
horizDrift:
	.word 0
powerUP:
	.word 0
powerUPDelay:
	.word 0
LevelEnd:
	.word 0
delaySF:
	.word 2
delaySB:
	.word 4
score:
	.word 0,0
adder:
	.word 0,0

vofsMain:
	.word 192
vofsSFMain:
	.word 192
vofsSBMain:
	.word 192
	
vofsSub:
	.word 0
vofsSFSub:
	.word 0
vofsSBSub:
	.word 0

yposMain:
	.word 896
yposSFMain:
	.word 832
yposSBMain:
	.word 832

yposSub:
	.word 896
yposSFSub:
	.word 832
yposSBSub:
	.word 832

xpos:
	.word 0
shipX:
	.byte 0
shipY:
	.byte 0
shipXspeed:
	.byte 0
	.byte 0


	@ sprite table!
	@ sprite 0 		= ship (spritetile 0)
	@ sprite 1-4	= Bullets (player spritetile 1)

spriteX:
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spriteY:
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spriteObj:
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spriteActive:
	.word 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spriteArmour:
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spriteSpeed:
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	
	.align 4
getReadyText:
	.byte 'G','E','T',' ','R','E','A','D','Y','!'
	
.end
